# ***************************************************************************
# Copyright 2017-2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import logging
import json

import cli.commands.host_shared.ovs_util as ovs_util
from cli.commands.cli_command_provider import CliCommandProvider
from cli.utils.node_utils import get_node_type
from cli.utils.constants import NODE_TYPE_HOST_CENTOSKVM
from cli.utils.constants import NODE_TYPE_HOST_SLESKVM
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.constants import NODE_TYPE_HOST_RHELKVM
from cli.utils.constants import NODE_TYPE_HOST_UBUNTUKVM
from cli.utils.constants import NODE_TYPE_HOST_WIN
from cli.utils.constants import NODE_TYPE_HOST_PUBLIC_CLOUD
from cli.utils.constants import NODE_TYPE_BAREMETAL_UBUNTU_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_SLES_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_OEL_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_WINDOWS_SERVER
from cli.utils.constants import NODE_TYPE_BAREMETAL_RHEL_CONTAINER
from cli.utils.constants import NODE_TYPE_BAREMETAL_CENTOS_CONTAINER
from cli.utils.error_code import ErrorCode as ec

lg = logging.getLogger("cli.commands.host_shared.switch_ipfix")


def has_switch_ipfix_cli(node_type):
    return node_type in (NODE_TYPE_BAREMETAL_UBUNTU_SERVER,
                         NODE_TYPE_BAREMETAL_RHEL_SERVER,
                         NODE_TYPE_BAREMETAL_CENTOS_SERVER,
                         NODE_TYPE_BAREMETAL_SLES_SERVER,
                         NODE_TYPE_BAREMETAL_OEL_SERVER,
                         NODE_TYPE_BAREMETAL_WINDOWS_SERVER,
                         NODE_TYPE_BAREMETAL_RHEL_CONTAINER,
                         NODE_TYPE_BAREMETAL_CENTOS_CONTAINER,
                         NODE_TYPE_HOST_CENTOSKVM,
                         NODE_TYPE_HOST_SLESKVM,
                         NODE_TYPE_HOST_RHELKVM,
                         NODE_TYPE_HOST_PUBLIC_CLOUD,
                         NODE_TYPE_HOST_UBUNTUKVM,
                         NODE_TYPE_HOST_WIN)


def invoke_switch_ipfix_command(id, is_lswitch = True):
    node_type = get_node_type()
    if has_switch_ipfix_cli(node_type):
        setting = get_ipfix_setting(id, is_lswitch)
        return (0, setting)
    else:
        lg.error("node type %s does not have switch ipfix CLI", node_type,
                 error_code=ec.INVALID_ARGUMENT)
        return (-1, None)


def get_ipfix_setting(id, is_lswitch = True):
    if is_lswitch:
        raw_data = ovs_util.get_lswitch_ipfix_config(id)
    else:
        raw_data = ovs_util.get_lport_ipfix_config(id)

    if raw_data is None:
        return None

    lines =  len(raw_data)
    if lines == 0:
        return None

    setting = {}
    setting['obsDomainID'] = raw_data[-1][1]
    setting['activeTimeout'] = raw_data[-1][2]
    setting['maxFlow'] = raw_data[-1][3]
    setting['sampleProbability'] = raw_data[-1][4]
    setting['vNICFlowOnly'] = raw_data[-1][5]
    setting['collectors'] = []
    for i in range(0, lines-2):
        setting['collectors'].append(raw_data[i][0] + ' ' + raw_data[i][1])

    return setting


class HostGetIpfixLogicalSwitchConfig(CliCommandProvider):
    def __call__(self, cmd_list):
        lswitch_id = cmd_list[3]
        rc, setting = invoke_switch_ipfix_command(lswitch_id, True)
        if rc == -1:
            return "MSG_SWITCHIPFIX_NOT_AVAILABLE"
        if setting is None:
            return "MSG_IPFIX_SETTING_NOT_FOUND"

        return "OK\n\n%s" % json.dumps(setting)


class HostGetIpfixLogicalPortConfig(CliCommandProvider):
    def __call__(self, cmd_list):
        lport_id = cmd_list[3]
        rc, setting = invoke_switch_ipfix_command(lport_id, False)
        if rc == -1:
            return "MSG_SWITCHIPFIX_NOT_AVAILABLE"
        if setting is None:
            return "MSG_IPFIX_SETTING_NOT_FOUND"

        return "OK\n\n%s" % json.dumps(setting)
