# ***************************************************************************
# Copyright 2014-2020 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import os
import nsx_logging

from datetime import datetime
from cli.utils.command_utils import invoke_command
from cli.utils.error_code import ErrorCode as ec
from cli.utils.node_utils import is_windows


lg = nsx_logging.getLogger("cli.utils.system_utils")


def get_system_datetime():
    result = {}
    if is_windows():
        now = datetime.now()
        result["date"], result["time"] =\
            now.strftime("%a %b %d %Y_%H:%M:%S.%f").split('_')
        cmd = ["tzutil", "/g"]
        rc, out, err = invoke_command(cmd)
        if rc == 0:
            result["timezone"] = out
    else:
        BIN_DATE = "/" + os.path.join("bin", "date")
        cmd = [BIN_DATE, "+%N_%H:%M:%S.%%s_%Z_%a %b %d %Y"]
        rc, out, err = invoke_command(cmd)
        # We need to show milliseconds, however date only shows nanosecs,
        # so the date command above returns nanoseconds in the first position
        # which we parse out now
        if rc == 0:
            parts = out.split("_")
            result["time"] = parts[1] % parts[0][0:3]
            result["timezone"] = parts[2]
            result["date"] = parts[3].strip()

    if rc != 0:
        lg.error("Error calling '%s', rc=%s, err: %s", cmd, rc, err,
                 error_code=ec.OPERATION_FAILED)
        return None

    return result
