# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential
from vmware.nsx.sha import sha_pb2

import sha.config
from sha.contrib.action.abc_base import Action
from sha.core.channel.provider.mpa_provider import ReportMsg

from sha.constants import REPORT_INTERVAL, SMALLEST_REPORT_INTERVAL_IF_CHANGE,\
    SRC_IP, DST_NODE, DST_IP, STATUS, LATENCY, LATENCY_LIST,\
    PACKET_TRANSMITTED, PACKET_RECEIVED, PACKET_LOSS_PERCENT, RTT_MIN, RTT_AVG,\
    RTT_MAX, RTT_MDEV, TOTAL_TIME


class UaNLReport(Action):
    _logger = sha.config.get_logger(__name__)

    @classmethod
    def populate_pb_msg(cls, **kwargs):
        report_msg = sha_pb2.ShaReportMsg()
        report_msg.oid.left = sha.config.HOST_ID_LOW
        report_msg.oid.right = sha.config.HOST_ID_HIGH
        latency_list = kwargs[LATENCY_LIST] or []
        status_msg = report_msg.status.add()
        status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value('LATENCY')
        for status in latency_list[:-1]:
            latency_msg = status_msg.latency_status.status.add()
            latency_msg.src_ip = status[SRC_IP]
            latency_msg.dst_ip = status[DST_IP]
            host_id = int(status[DST_NODE].replace('-', ''), 16)
            latency_msg.dst_node.left = (host_id >> 64) & 0xffffffffffffffff
            latency_msg.dst_node.right = host_id & 0xffffffffffffffff
            if PACKET_TRANSMITTED in status:
                latency_msg.packet_transmitted = status[PACKET_TRANSMITTED]
            if PACKET_RECEIVED in status:
                latency_msg.packet_received = status[PACKET_RECEIVED]
            if PACKET_LOSS_PERCENT in status:
                latency_msg.packet_loss_percent = status[PACKET_LOSS_PERCENT]
            if TOTAL_TIME in status:
                latency_msg.total_time = status[TOTAL_TIME]
            if RTT_MIN in status:
                latency_msg.rtt_min = status[RTT_MIN]
            if RTT_AVG in status:
                latency_msg.rtt_avg = status[RTT_AVG]
            if RTT_MAX in status:
                latency_msg.rtt_max = status[RTT_MAX]
            if RTT_MDEV in status:
                latency_msg.rtt_mdev = status[RTT_MDEV]
            if STATUS in status:
                latency_msg.status = status[STATUS]

        return report_msg.SerializeToString()

    @classmethod
    def do(cls, hub, *args):
        def kernel(**kwargs):
            cls._logger.debug('UaNodeLatencyReport - signaling, connection_list:{}, profile:{}'
                              .format(kwargs['latency_list'], kwargs['profile']))

            pb_msg = cls.populate_pb_msg(**kwargs)
            cls._logger.debug("pb_msg: {}".format(pb_msg))
            hub.send(ReportMsg(b'', pb_msg, b'com.vmware.nsx.sha.ShaReportMsg'))

        return kernel
