# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential
import sha.config as conf
from sha.contrib.metric.abc_base import Metric
import json
import re


class EdgeMgmtInterface(Metric):
    def __init__(self):
        self._logger = conf.get_logger(__name__)

    def get_mgmt_interface_name(self):
        try:
            # First: read from config.json
            with open("/config/vmware/edge/config.json", "r") as dp_conf_file:
                dp_conf = json.load(dp_conf_file)
                if 'custom' in dp_conf and 'mgmt_dev_name' in dp_conf['custom']:
                    self._logger.debug("Got mgmt interface from config")
                    return dp_conf['custom']['mgmt_dev_name']
            # Second: got from default route
            pattern = re.compile("^\s*([^\s]+)\s+0+\s+")
            with open("/proc/net/route", "r") as route_file:
                for entry in route_file:
                    m = pattern.match(entry)
                    if m:
                        self._logger.debug("Got mgmt interface via default gw")
                        return m.group(1)
        except Exception as e:
            self._logger.warn("Failed to get mgmt interface: %s", e)

        # Finally: return default "eth0"
        self._logger.debug("Got default mgmt interface")
        return "eth0"

    def usage(self, *args, **kwargs):
        if not conf.node_type_is(conf.DIST_EDGE,
                                 conf.DIST_AUTONOMOUS_EDGE,
                                 conf.DIST_PUBLIC_CLOUD_GATEWAY):
            return None

        intf_name = self.get_mgmt_interface_name()
        status = {
            "interface_name": intf_name,
            "op_state": "down",
        }
        try:
            with open("/sys/class/net/{}/operstate".format(intf_name), "r") as status_file:
                status["op_state"] = status_file.read().strip().lower()
        except:
            pass

        self._logger.debug("Got the status of edge mgmt interface: %s", status)
        return status
