# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential
import json
import subprocess
import traceback

import sha.config
from sha.contrib.metric.abc_base import Metric
from sha.contrib.metric.utils import run_command, timeout_cache
# from vmware.nsx.sha import sha_pb2

ms_logger = sha.config.get_logger(__name__)


class MirrorStackStatus(Metric):
    # This flag is used to indicate whether exist mirror session on
    # host, if exist to set it to True, else to set it to False
    change_flag = False

    def __init__(self):
        self.vmknic_number = 0
        self.description = "Unknown"

    @timeout_cache(2)
    def usage(self, *args, **kwargs):

        UNKNOWN = 0
        HEALTHY = 1
        UNHEALTHY = 2

        def get_l3span_with_stack():
            """
            check mirror sesion has been configuread L3SPAN and enabled
            mirror stack
            """
            ms_ids = set()
            cmd_l3span_with_stack = ['/opt/vmware/nsx-cli/bin/nsx-appctl', '-t',
                    '/var/run/vmware/nsx-cfgagent/cfgagent-cli', 'cfgagent/config/mirror']
            # exist l3span_mirror_session
            # and mirror_stack to be true
            ms_logger.debug('check l3span exist with mirror stack{}'
                            .format(cmd_l3span_with_stack))
            output = run_command(cmd_l3span_with_stack, shell=False)
            raw_sessions = json.loads(output)

            for session in raw_sessions:
                if "mirror_stack: true" in session["PropertyValue"]:
                    ms_logger.debug("config l3span {} with mirror stack",
                                   session["ObjectID"])
                    ms_ids.add(session["ObjectID"])
                    MirrorStackStatus.change_flag = True
            ms_logger.debug('got ms uuids: {}'.format(ms_ids))
            return ms_ids

        def mirror_stack_health_status():
            # stack status
            cmd_mirror_stack = ['localcli', '--formatter=json', 'network',
                    'ip', 'interface', 'list', '-N', 'mirror']
            ms_logger.debug('check mirror stack health status \
                            {}'.format(cmd_mirror_stack))

            # if throw exception, the mirror stack not exist
            try:
                health_result = run_command(cmd_mirror_stack, shell=False)
            except subprocess.CalledProcessError as e:
                trace = traceback.format_exc()
                ms_logger.error('Failed in getting mirror stack health status'
                                   'check stack whether exitst, cmd:{}, '
                                   'exception:{}, trace:{}'.format(cmd_mirror_stack, e, trace))
                self.description = "No mirror stack"
                return UNHEALTHY, UNHEALTHY

            health_result = json.loads(health_result)
            for status in health_result:
                self.vmknic_number += 1
                if status["Enabled"]:
                    self.vmknic_number = 0;
                    self.description = "Both mirror stack and vmknic are HEALTHY"
                    return HEALTHY, HEALTHY
                else:
                    continue

            if (self.vmknic_number > 0):
                self.description = "vmknics are disabled"
                return HEALTHY, UNHEALTHY
            else:
                self.description = "No vmknic"
                return HEALTHY, UNHEALTHY,

        ms_ids = get_l3span_with_stack()
        ms_status = {}
        if (ms_ids):
            ms_status["dedicated_stack_status"], ms_status["vmknic_status"] =\
                mirror_stack_health_status()
            ms_status["session_ids"] = sorted(ms_ids)
            ms_status["description"] = self.description
        else:
            if (MirrorStackStatus.change_flag):
                ms_status["dedicated_stack_status"], ms_status["vmknic_status"] =\
                    UNKNOWN, UNKNOWN
                ms_status["session_ids"] = ms_ids
                ms_status["description"] = self.description
                MirrorStackStatus.change_flag = False

        return ms_status
