# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential
import os
import traceback

import sha.config
from sha.contrib.metric.abc_base import Metric
from sha.contrib.metric.utils import run_command, timeout_cache


class VMKList(Metric):

    def __init__(self):
        self._logger = sha.config.get_logger(__name__)

    @timeout_cache(2)
    def usage(self, *args, **kwargs):
        uname = os.uname()
        if uname.sysname != "VMKernel":
            # Non-VMK platforms do not support the vmknic cmd and also don't have
            # vmk50 port too, so just return a fake one
            return ["vmk50"]
        else:
            # TODO: nobody consume this plugin now. Need convert command to fit
            # "shell=False" format when someone pick it up.
            cmd = """esxcfg-vmknic -l|cut -d ' ' -f 1"""
            output = run_command(cmd)
            vmk_set = set()
            try:
                for vmk_name in output.strip().split()[1:]:
                    vmk_set.add(vmk_name)
            except Exception as e:
                trace = traceback.format_exc()
                self._logger.error("can not retrieve {}, exception:{}, trace:{}"
                                   .format(self.__class__.__name__, e, trace))
            self._logger.debug('got list {}'.format(vmk_set))
            return list(vmk_set)
