# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential

def report_edge_fw_conn(edge_fw_stats, edge_fw_conn, profile):
    if edge_fw_conn is None and edge_fw_stats is None:
        return


    if edge_fw_stats is not None and edge_fw_stats['value'] is not None:
        # Use specified source and timestamp to export metrics to wavefront
        wavefront_source = edge_fw_stats['value'][0]
        timestamp = edge_fw_stats['timestamp']

        # Get firewall ingress conn info from stats
        fw_stats_sum = {}
        for lr, fw_stats in edge_fw_stats['value'][1].items():
            if fw_stats is None:
                continue

            stats_conn_data = {}
            fw_stats_name = "edge." + lr + ".firewall.connections"

            stats_conn_data['tcp'] = {'halfOpenIngress': fw_stats['TCP Half Opened Active/Max'].split('/')[0],
                                      'max': fw_stats['TCP Half Opened Active/Max'].split('/')[1]}
            stats_conn_data['others'] = {'ingress': fw_stats['Other Active/Max'].split('/')[0],
                                         'max': fw_stats['Other Active/Max'].split('/')[1]}
            for item in ['ICMP', 'UDP']:
                stats_conn_data[item.lower()] = {'ingress': fw_stats[item + ' Active/Max'].split('/')[0],
                                                     'max': fw_stats[item + ' Active/Max'].split('/')[1]}

            fw_stats_sum[fw_stats_name] = stats_conn_data

        # export conn data from edge_fw_stats
        if fw_stats_sum:
            Exportdata(data = fw_stats_sum, exports = ['wavefront'],
                       source = wavefront_source, timestamp = timestamp)

    if edge_fw_conn is None or edge_fw_conn['value'] is None:
        return

    # Use specified source and timestamp to export metrics to wavefront
    wavefront_source = edge_fw_conn['value'][0]
    timestamp = edge_fw_conn['timestamp']

    for lr, fw_conn in edge_fw_conn['value'][1].items():

        stats_data = {}
        stats_conn_data = {}
        fw_stats_name = "edge." + lr + ".firewall"

        # firewall conn
        if fw_conn is not None:
            if 'estConn' in fw_conn:
                for key, value in fw_conn['estConn'].items():
                    if key in stats_conn_data:
                        stats_conn_data[key]['established'] = value
                    else:
                        stats_conn_data[key] = {'established': value}

            if 'tcp' in fw_conn:
                if 'tcp' in stats_conn_data:
                    stats_conn_data['tcp']['open'] = fw_conn['tcp']
                else:
                    stats_conn_data['tcp'] = {'open': fw_conn['tcp']}

            if 'flowsPerCore' in fw_conn:
                stats_data['flowsAndCores'] = fw_conn['flowsPerCore']


        if stats_conn_data:
           stats_data['connections'] = stats_conn_data

        if stats_data:
            Exportdata(data = {fw_stats_name: stats_data}, exports = ['wavefront'],
                               source = wavefront_source, timestamp = timestamp)


profile = data_store['profile']
edge_fw_stats = data_store['metrics']['nsx.edge.edge-fw-stats']
edge_fw_conn = data_store['metrics']['nsx.edge.edge-fw-conn']
report_edge_fw_conn(edge_fw_stats, edge_fw_conn, profile)
